/*
 *  Copyright 2011 StripBandunk.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package com.stripbandunk.webporting.engine;

import java.io.IOException;
import java.io.OutputStream;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.stripbandunk.webporting.error.AccessDeniedException;
import com.stripbandunk.webporting.error.ReportException;
import com.stripbandunk.webporting.helper.WebportingExporterHelper;
import com.stripbandunk.webporting.type.ReportType;
import com.stripbandunk.webporting.type.WebMethodType;
import com.stripbandunk.webporting.http.ReportRequest;
import com.stripbandunk.webporting.http.ReportResponse;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperPrint;

/**
 *
 * @author Eko Kurniawan Khannedy
 */
public abstract class WebportingEngine extends HttpServlet implements WebportingEngineConstant {

    private static final long serialVersionUID = 1L;

    public abstract void beforeProcess(ReportRequest request, ReportResponse response) throws ServletException, IOException;

    public abstract void afterProcess(ReportRequest request, ReportResponse response) throws ServletException, IOException;

    public abstract String getFileName(ReportRequest request, ReportResponse response) throws ServletException, IOException;

    public abstract ReportType getReportType(ReportRequest request, ReportResponse response) throws ServletException, IOException;

    public abstract boolean isDenied(ReportRequest request, ReportResponse response) throws ServletException, IOException;

    public abstract JasperPrint getJasperPrint(ReportRequest request, ReportResponse response) throws ServletException, IOException, JRException;

    protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        ServletContext context = getServletContext();
        WebportingEngineConverter webportingConverter = (WebportingEngineConverter) context.getAttribute(WEBPORTING_CONVERTER);

        ReportRequest reportRequest = new ReportRequest(request, webportingConverter);
        ReportResponse reportResponse = new ReportResponse(response);

        if (isDenied(reportRequest, reportResponse)) {
            onDenied(reportRequest, reportResponse);
        } else {
            ReportType reportType = getReportType(reportRequest, reportResponse);
            if (isForceDownload(reportRequest, reportResponse)) {
                response.setContentType("application/force-download");
            } else {
                response.setContentType(reportType.getContentType());
            }

            String fileName = getFileName(reportRequest, reportResponse);
            response.setHeader("Content-disposition", "inline; filename=\"" + fileName + "\"");

            OutputStream stream = reportResponse.getOutputStream();

            try {
                beforeProcess(reportRequest, reportResponse);
                JasperPrint print = getJasperPrint(reportRequest, reportResponse);
                WebportingExporterHelper.export(print, reportType, stream);
            } catch (JRException ex) {
                throw new ReportException(ex);
            } finally {
                afterProcess(reportRequest, reportResponse);
                stream.close();
            }
        }
    }

    //<editor-fold defaultstate="collapsed" desc="Do Web Method">
    @Override
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.DELETE)) {
            processRequest(req, resp);
        }
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.GET)) {
            processRequest(req, resp);
        }
    }

    @Override
    protected void doHead(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.HEAD)) {
            processRequest(req, resp);
        }
    }

    @Override
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.OPTIONS)) {
            processRequest(req, resp);
        }
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.POST)) {
            processRequest(req, resp);
        }
    }

    @Override
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.PUT)) {
            processRequest(req, resp);
        }
    }

    @Override
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (isSupport(WebMethodType.TRACE)) {
            processRequest(req, resp);
        }
    }
    //</editor-fold>

    public boolean isForceDownload(ReportRequest request, ReportResponse response) throws ServletException, IOException {
        return false;
    }

    public boolean isSupport(WebMethodType type) {
        return type == WebMethodType.GET;
    }

    public void onDenied(ReportRequest request, ReportResponse response) throws ServletException, IOException {
        throw new AccessDeniedException();
    }
}
